/*
   Copyright (c) 1995-1996 by Cisco systems, Inc.

   Permission to use, copy, modify, and distribute this software for
   any purpose and without fee is hereby granted, provided that this
   copyright and permission notice appear on all copies of the
   software and supporting documentation, the name of Cisco Systems,
   Inc. not be used in advertising or publicity pertaining to
   distribution of the program without specific prior permission, and
   notice be given in supporting documentation that modification,
   copying and distribution is by permission of Cisco Systems, Inc.

   Cisco Systems, Inc. makes no representations about the suitability
   of this software for any purpose.  THIS SOFTWARE IS PROVIDED ``AS
   IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
   WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE.
*/

/* Keywords of the configuration language */

#include "tac_plus.h"

static void *wordtable[HASH_TAB_SIZE];	/* Table of keyword declarations */

struct keyword {
    char *word;
    void *hash;
    u_char value;
};

typedef struct keyword KEYWORD;

static void
declare(name, value)
    char *name;
    int value;
{
    KEYWORD *n;
    KEYWORD *k = (KEYWORD *)tac_malloc(sizeof(KEYWORD));

    k->word = tac_strdup(name);
    k->value = value;

    n = hash_add_entry(wordtable, (void *) k);

    if (n) {
	report(LOG_ERR, "Attempt to multiply define keyword %s",
	       name);
	tac_exit(1);
    }
}

/* Declare keywords of the "configuration language". */

void
parser_init()
{
    bzero(wordtable, sizeof(wordtable));

    declare("access", S_access);
    declare("accounting", S_accounting);
    declare("after", S_after);
    declare("arap", S_arap);
    declare("attribute", S_attr);
    declare("authentication", S_authentication);
    declare("authorization", S_authorization);
    declare("before", S_before);
    declare("chap", S_chap);
    declare("cleartext", S_cleartext);
    declare("cmd", S_cmd);
    declare("default", S_default);
    declare("deny", S_deny);
    declare("des", S_des);
#ifdef EMESSAGE
    declare("emessage",S_emessage);
#endif
#ifdef PERUSER_ENABLE
    declare("enable",S_enable);
#endif
    declare("exec", S_exec);
    declare("expires", S_expires);
    declare("file", S_file);
    declare("group", S_group);
    declare("global", S_global);
    declare("host", S_host);
    declare("ip", S_ip);
    declare("ipx", S_ipx);
    declare("key", S_key);
    declare("lcp", S_lcp);
#ifdef MAXSESS
    declare("maxsess", S_maxsess);
#endif
    declare("member", S_member);
    declare("message", S_message);
    declare("name", S_name);
#if defined(NONE_LOGIN) || defined(PERUSER_ENABLE)
    declare("none", S_none);
#endif
    declare("optional", S_optional);
    declare("login", S_login);
    declare("permit", S_permit);
    declare("pap", S_pap);
    declare("opap", S_opap);
    declare("ppp", S_ppp);
#ifdef UNIX_LOGIN
    declare("prompt", S_prompt);
#endif
    declare("protocol", S_protocol);
#ifdef UNIX_LOGIN
    declare("reject", S_reject);
#endif
    declare("skey", S_skey);
    declare("slip", S_slip);
    declare("service", S_svc);
    declare("user", S_user);
#if defined(AUTO_MEMBERS) || defined(COMMUNITY)
    declare("users", S_users);
#endif
#ifdef CRON
    declare("crondeny", S_crondeny);
#endif
#ifdef DENY
    declare("admdeny", S_admdeny);
#endif
#ifdef COMMUNITY
    declare("community", S_community);
    declare("strip", S_strip);
    declare("flags", S_flags);
    declare("unique", S_unique);
 #ifdef UTMP
    declare("utmp", S_utmp);
 #endif
#endif
#ifdef DYNA
    declare("dynip-server", S_dynip);
    declare("address-pool", S_apool);
#endif /* DYNA */
}

/* Return a keyword code if a keyword is recognized. 0 otherwise */
int
keycode(keyword)
char *keyword;
{
    KEYWORD *k = hash_lookup(wordtable, keyword);

    if (k)
	return (k->value);
    return (S_unknown);
}

char *
codestring(type)
int type;
{
    switch (type) {
    default:
	return ("<unknown symbol>");
    case S_eof:
	return ("end-of-file");
    case S_unknown:
	return ("unknown");
    case S_separator:
	return ("=");
    case S_string:
	return ("<string>");
    case S_openbra:
	return ("{");
    case S_closebra:
	return ("}");
    case S_key:
	return ("key");
    case S_user:
	return ("user");
    case S_group:
	return ("group");
    case S_host:
	return ("host");
    case S_file:
	return ("file");
    case S_skey:
	return ("skey");
    case S_name:
	return ("name");
#if defined(NONE_LOGIN) || defined(PERUSER_ENABLE)
    case S_none:
	return ("none");
#endif
    case S_login:
	return ("login");
#ifdef EMESSAGE
    case S_emessage:
        return("emessage");
#endif
#ifdef PERUSER_ENABLE
    case S_enable:
        return("enable");
#endif
    case S_member:
	return ("member");
#if defined(AUTO_MEMBERS) || defined(COMMUNITY)
    case S_users:
	return ("users");
#endif
#ifdef MAXSESS
    case S_maxsess:
	return ("maxsess");
#endif
    case S_expires:
	return ("expires");
    case S_after:
	return ("after");
    case S_before:
	return ("before");
    case S_message:
	return ("message");
    case S_arap:
	return ("arap");
    case S_global:
	return ("global");
    case S_chap:
	return ("chap");
    case S_pap:
	return ("pap");
    case S_opap:
	return ("opap");
    case S_cleartext:
	return ("cleartext");
    case S_des:
	return("des");
    case S_svc:
	return ("service");
    case S_default:
	return ("default");
    case S_access:
	return ("access");
    case S_deny:
	return ("deny");
    case S_permit:
	return ("permit");
    case S_exec:
	return ("exec");
#ifdef UNIX_LOGIN
    case S_prompt:
	return ("prompt");
#endif
    case S_protocol:
	return ("protocol");
#ifdef UNIX_LOGIN
    case S_reject:
	return ("reject");
#endif
    case S_optional:
	return ("optional");
    case S_ip:
	return ("ip");
    case S_ipx:
	return ("ipx");
    case S_slip:
	return ("slip");
    case S_ppp:
	return ("ppp");
    case S_authentication:
	return ("authentication");
    case S_authorization:
	return ("authorization");
    case S_cmd:
	return ("cmd");
    case S_attr:
	return ("attribute");
    case S_svc_dflt:
	return ("svc_dflt");
    case S_accounting:
	return ("accounting");
    case S_lcp:
	return("lcp");
#ifdef DENY
    case S_admdeny:
	return ("admdeny");
#endif
#ifdef CRON
    case S_crondeny:
	return ("crondeny");
#endif
#ifdef COMMUNITY
    case S_community:
	return ("community");
    case S_strip:
	return ("strip");
    case S_flags:
	return ("flags");
    case S_unique:
	return ("unique");
 #ifdef UTMP
    case S_utmp:
	return ("utmp");
 #endif
#endif
#ifdef DYNA
    case S_dynip:
        return ("dynip-server");
    case S_apool:
        return ("address-pool");
#endif /* DYNA */
    }
}
